import type { PeerId } from "@libp2p/interface-peer-id";
import type { ConnectionManager } from "@libp2p/interface-internal/connection-manager";
import type { Registrar } from "@libp2p/interface-internal/registrar";
import type { PubSub } from "@libp2p/interface/pubsub";
import type { Startable } from "@libp2p/interface/startable";
import { EventEmitter } from "@libp2p/interface/events";
import type { Signature } from "@canvas-js/signed-cid";
import type { Message } from "@canvas-js/interfaces";
import { AbstractMessageLog, MessageLogEvents, MessageSigner } from "./AbstractMessageLog.js";
import { SyncOptions } from "./SyncService.js";
import { Awaitable } from "./utils.js";
export type GossipLogComponents = {
    peerId: PeerId;
    registrar: Registrar;
    connectionManager: ConnectionManager;
    pubsub?: PubSub;
};
export interface GossipLogInit {
    sync?: boolean;
}
export interface TopicInit<Payload, Result> extends SyncOptions {
    apply: (id: string, signature: Signature | null, message: Message<Payload>) => Awaitable<Result>;
    validate: (payload: unknown) => payload is Payload;
    signatures?: boolean;
    sequencing?: boolean;
    replay?: boolean;
}
export type GossipLogEvents = MessageLogEvents<unknown, unknown>;
export declare class GossipLog extends EventEmitter<GossipLogEvents> implements Startable {
    #private;
    private readonly components;
    private readonly sync;
    private readonly log;
    constructor(components: GossipLogComponents, init: GossipLogInit);
    isStarted(): boolean;
    start(): Promise<void>;
    afterStart(): Promise<void>;
    beforeStop(): Promise<void>;
    stop(): Promise<void>;
    subscribe<Payload, Result>(messageLog: AbstractMessageLog<Payload, Result>, options?: SyncOptions): Promise<void>;
    unsubscribe(topic: string): Promise<void>;
    private forwardEvent;
    append<Payload, Result>(topic: string, payload: Payload, options?: {
        signer?: MessageSigner<Payload>;
    }): Promise<{
        id: string;
        result: Result;
        recipients: Promise<PeerId[]>;
    }>;
    insert<Payload, Result = unknown>(topic: string, signature: Signature | null, message: Message<Payload>): Promise<{
        id: string;
        recipients: Promise<PeerId[]>;
    }>;
    getTopics(): string[];
    private handleMessage;
}
export declare const gossiplog: (init: GossipLogInit) => (components: GossipLogComponents) => GossipLog;
