import type { PeerId } from "@libp2p/interface-peer-id";
import type { Source, Target, Node } from "@canvas-js/okra";
import { Logger } from "@libp2p/logger";
import type { Message } from "@canvas-js/interfaces";
import { Signature } from "@canvas-js/signed-cid";
import { Awaitable } from "../utils.js";
export interface ReadOnlyTransaction extends Target {
    get(key: Uint8Array): Awaitable<Uint8Array | null>;
    getUserdata(): Awaitable<Uint8Array | null>;
}
export interface ReadWriteTransaction extends ReadOnlyTransaction {
    set(key: Uint8Array, value: Uint8Array): Awaitable<void>;
    delete(key: Uint8Array): Awaitable<void>;
    setUserdata(userdata: Uint8Array | null): Awaitable<void>;
}
export interface StoreInit {
    topic: string;
    location: string | null;
    signatures: boolean;
    sequencing: boolean;
}
export declare abstract class AbstractStore {
    private readonly init;
    abstract close(): Promise<void>;
    abstract source(targetPeerId: PeerId, callback: (txn: ReadOnlyTransaction) => Promise<void>): Promise<void>;
    abstract target(sourcePeerId: PeerId, callback: (txn: ReadWriteTransaction) => Promise<void>): Promise<void>;
    abstract read<T>(callback: (txn: ReadOnlyTransaction) => Promise<T>): Promise<T>;
    abstract write<T>(callback: (txn: ReadWriteTransaction) => Promise<T>): Promise<T>;
    protected readonly log: Logger;
    protected constructor(init: StoreInit);
    get(key: Uint8Array): Promise<[signature: Signature | null, message: Message | null]>;
    sync(peerId: PeerId, source: Source, callback: (key: Uint8Array, signature: Signature | null, message: Message) => Promise<void>): Promise<{
        root: Node;
    }>;
}
export declare class Graph {
    #private;
    private readonly log;
    static import(txn: ReadOnlyTransaction): Promise<Graph>;
    constructor(parents: Uint8Array[]);
    save(txn: ReadWriteTransaction): Promise<void>;
    update(key: Uint8Array, message: Message): void;
    export(): [clock: number, parents: Uint8Array[]];
}
