import path from "node:path";
import fs from "node:fs";
import { ReadOnlyTransaction, Tree } from "@canvas-js/okra-node";
import { assert } from "@canvas-js/okra";
import openMemoryMessageLog from "../memory/index.js";
import { AbstractMessageLog } from "../AbstractMessageLog.js";
export * from "../AbstractMessageLog.js";
export default async function openMessageLog(init) {
    if (init.location === null) {
        return openMemoryMessageLog(init);
    }
    const directory = path.resolve(init.location, init.topic);
    if (!fs.existsSync(directory)) {
        fs.mkdirSync(directory, { recursive: true });
    }
    const tree = new Tree(directory);
    return new MessageLog(init, tree);
}
class MessageLog extends AbstractMessageLog {
    constructor(init, tree) {
        super(init);
        this.tree = tree;
    }
    async close() {
        await this.tree.close();
    }
    async *entries(lowerBound = null, upperBound = null, options = {}) {
        const txn = new ReadOnlyTransaction(this.tree);
        try {
            for await (const node of txn.nodes(0, lowerBound ?? { key: null, inclusive: false }, upperBound, options)) {
                assert(node.key !== null, "expected node.key !== null");
                assert(node.value !== undefined, "expected node.value !== undefined");
                yield [node.key, node.value];
            }
        }
        finally {
            txn.abort();
        }
    }
    async read(callback) {
        return await this.tree.read((txn) => callback(txn));
    }
    async write(callback) {
        return await this.tree.write((txn) => callback(txn));
    }
}
